class Event:
    def __init__(self, time_str='', type_name='', actor_id='', action_name=''):
        self.__time = time_str
        self.__type = type_name
        self.__actor = actor_id
        self.__action = action_name

    @property
    def time(self):
        return self.__time

    @time.setter
    def time(self, time_string: str):
        self.__time = time_string

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, type_name: str):
        self.__type = type_name

    @property
    def actor(self):
        return self.__actor

    @actor.setter
    def actor(self, actor_id: str):
        self.__actor = actor_id

    @property
    def action(self):
        return self.__action

    @action.setter
    def action(self, action_name):
        self.__action = action_name



"""
Main application classes. Used to initialize the application.
"""
import sys

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication
from docker import DockerClient

from events_dock.components import Events
from events_dock.threads import EventsThread
from events_dock.utils import Config


class App:
    def __init__(self, client: DockerClient):
        self.__config = Config()
        self.__app = QApplication(sys.argv)
        self.__client = client
        self.__main_widget = Events(client)

    def run(self):
        self.__main_widget.show()

        events_thread = EventsThread(self.__client)
        events_thread.start(priority=1)
        events_thread.docker_event.connect(self.__main_widget.on_event)

        self.__app.exec()

    @property
    def client(self):
        return self.__client



from events_dock.models import Event


class EventMapper:
    @staticmethod
    def map(event_data: dict):
        """
        :param event_data: Event data from event stream.
        :return Event: Returns Event object.
        """
        event = Event(
            time_str=event_data.get('timeNano'),
            type_name=event_data.get('Type'),
            actor_id=event_data.get('Actor').get('ID'),
            action_name=event_data.get('Action')
        )

        return event



import configparser
import os


class Config:
    def __init__(self):
        path = os.path.dirname(os.path.abspath(__file__))
        self.__config = configparser.ConfigParser()
        self.__config.read(path + '/config.ini')
        self.__config.sections()

    def get(self, key: str):
        (section, value) = key.split('.')

        return self.__config[section][value]

    def section(self, section: str):
        return self.__config[section]



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from os import path

from setuptools import setup, find_packages

from events_dock.utils import Config

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='events_dock',
    version=Config().get('app.version'),
    description='Application used to control docker events.',
    author='Todor Todorov',
    author_email='todstoychev@gmail.com',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='docker containers control',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['docker', 'PyQt5', 'QtAwesome', 'events'],
    package_data={
        '': ['config.ini']
    },
    entry_points={
        'console_scripts': [
            'events-dock=events_dock.__main__:main',
        ],
    }
)



import json
from datetime import datetime

import qtawesome
from PyQt5.QtCore import pyqtSlot, Qt, QDateTime
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QTableWidget, QWidget, QTableWidgetItem, QAbstractItemView, QDateTimeEdit, \
    QLabel, QVBoxLayout, QHBoxLayout, QPushButton, QCalendarWidget
from docker import DockerClient

from events_dock.mappers import EventMapper
from events_dock.models import Event
from events_dock.utils import Config


class QVboLayout(object):
    pass


class Events(QWidget):
    stream_events = True

    def __init__(self, client: DockerClient, flags=None, *args, **kwargs):
        super().__init__(flags, *args, **kwargs)
        self.__client = client
        self.__layout = QVBoxLayout()
        self.__events_table = EventsTable()
        self.__config = Config()
        self.__start_date = DateTimePicker()
        self.__end_date = DateTimePicker()
        self.__filter = QPushButton('Filter')
        self.__clear = QPushButton('Clear')
        title = self.__config.get('app.name') + ' v.' + self.__config.get('app.version')
        self.setWindowTitle(title)
        self.setWindowIcon(QIcon(self.__config.get('app.icon')))

    def show(self):
        # Filter form
        self.setLayout(self.__layout)
        row = QHBoxLayout()
        row.addWidget(QLabel('From:'), 0, Qt.AlignLeft)
        row.addWidget(self.__start_date, 0, Qt.AlignLeft)
        row.addWidget(QLabel('To:'), 0, Qt.AlignLeft)
        row.addWidget(self.__end_date, 0, Qt.AlignLeft)
        self.__filter.setIcon(QIcon(qtawesome.icon('fa.filter')))
        row.addWidget(self.__filter, 1, Qt.AlignLeft)
        self.__clear.setIcon(qtawesome.icon('fa.remove'))
        row.addWidget(self.__clear, 2, Qt.AlignLeft)
        self.__start_date.setDateTime(QDateTime().currentDateTime().addDays(-1))
        self.__end_date.setDateTime(QDateTime().currentDateTime())
        self.__end_date.setMinimumDateTime(self.__start_date.dateTime())

        # Filter events
        self.__start_date.dateChanged.connect(self.on_start_date_changed)
        self.__clear.clicked.connect(self.on_clear)
        self.__filter.clicked.connect(self.on_filter)

        self.__layout.addLayout(row)
        self.__layout.addWidget(self.__events_table)
        self.__events_table.show()
        width = self.__events_table.width()
        self.setMinimumWidth(width)

        return super().show()

    @pyqtSlot(Event, name='docker_event')
    def on_event(self, event):
        if Events.stream_events:
            self.__events_table.on_event(event)

    @pyqtSlot(name="clear_filter")
    def on_clear(self):
        self.__events_table.setRowCount(0)
        self.__start_date.setDateTime(QDateTime().currentDateTime().addDays(-1))
        self.__end_date.setDateTime(QDateTime().currentDateTime())
        Events.stream_events = True

    @pyqtSlot(name='start_date_changed')
    def on_start_date_changed(self):
        date_time = self.__start_date.dateTime()
        self.__end_date.setMinimumDateTime(date_time)

    @pyqtSlot(Event, name='docker_event')
    def on_filter_event(self, event: Event):
        print(event)
        self.__events_table.on_event(event)

    @pyqtSlot(name='filter')
    def on_filter(self):
        Events.stream_events = False
        self.__events_table.setRowCount(0)
        since = self.__start_date.dateTime().toSecsSinceEpoch()
        until = self.__end_date.dateTime().toSecsSinceEpoch()
        data = self.__client.events(until=until, since=since)

        for event in data:
            event = json.loads(event.decode('utf-8'))
            event = EventMapper.map(event)
            self.__events_table.on_event(event)

        data.close()


class EventsTable(QTableWidget):
    def __init__(self, *__args):
        self.__config = Config()
        super().__init__(*__args)

    def show(self):
        self.setColumnCount(self.__config.section('table').__len__())
        self.setSortingEnabled(True)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        columns = []

        for column in self.__config.section('table'):
            columns.append(self.__config.get('table.' + column))

        self.setHorizontalHeaderLabels(columns)

        return super().show()

    def on_event(self, event: Event):
        position = self.rowCount()
        time = datetime.fromtimestamp(event.time / 1000000000)
        self.insertRow(position)
        self.setItem(position, 0, QTableWidgetItem(time.__str__()))
        self.setItem(position, 1, QTableWidgetItem(event.action))
        self.setItem(position, 2, QTableWidgetItem(event.type))
        self.setItem(position, 3, QTableWidgetItem(event.actor))
        self.resizeColumnToContents(0)
        self.scrollToBottom()


class DateTimePicker(QDateTimeEdit):
    def __init__(self, *__args):
        super().__init__(*__args)
        self.setCalendarPopup(True)
        calendar = QCalendarWidget()
        calendar.setGridVisible(True)
        calendar.setFirstDayOfWeek(Qt.Monday)
        self.setCalendarWidget(calendar)
        self.setDisplayFormat('d MMM yyyy - hh:mm:ss')


class ClearButton(QPushButton):
    def __init__(self, *__args):
        super().__init__(*__args)
        self.setIcon(qtawesome.icon('fa.remove'))
        self.setText('Clear')
        self.setToolTip('Clear filter')

    def show(self):
        super().show()


class FilterButton(QPushButton):
    def __init__(self, *__args):
        super().__init__(*__args)
        self.setIcon(qtawesome.icon('fa.filter'))
        self.setToolTip('Filter events results.')
        self.setText('Filter')

    def show(self):
        super().show()
